/******************************************************************************
*                                                                             *
*   B a c k u p   S y s t e m   S e t t i n g s                          			*
*                                                                             *
*   by Stefan Heule																														*
*   member of boolsoft (www.boolsoft.org)																			*
*                                                                             *
*   Source Code			                                                          *
*                                                                             *
*   File:   main.c                                                            *
*   Use:    - source code file                                                *
*           - main routine                                                    *
*                                                                             *
*******************************************************************************

Backup System Settings - Backups all settings on TI's 68k calculators
Copyright (C) 2007-2008  Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

// ===============================================================
// custom events
// ===============================================================
#define CUSTOM_CM_BACKUP_SETTINGS_ALL_TSR   0x6000
#define CUSTOM_CM_RESTORE_SETTINGS_ALL_TSR  0x6001
#define CUSTOM_CM_BACKUP_SETTINGS_AMSEXT    0x6002
#define CUSTOM_CM_RESTORE_SETTINGS_AMSEXT   0x6003
#define CUSTOM_CM_BACKUP_SETTINGS_EASYCHAR  0x6004
#define CUSTOM_CM_RESTORE_SETTINGS_EASYCHAR 0x6005



// ===============================================================
// required includes																							
// ===============================================================
#include <tigcclib.h>


// ===============================================================
// defines
// ===============================================================
#define COMMENT_AUTHORS          "Stefan Heule"
#define COMMENT_PROGRAM_NAME     "BSS"
#define COMMENT_VERSION_STRING   "1.1.0"

// if you change the backup struct, change this macig, too!
#define BACKUP_FILE_MAGIC        "bsscfg01"

// smaller code
#define NO_CALC_DETECT

#define fatal(s) {ST_showHelp(s); exit(0);}

#define DateAndTime_SetDlg _rom_call(void,(void),5F1)
#define ClockOn _rom_call(void,(void),5F5)
#define ClockOff _rom_call(void,(void),5F6)
#define IsClockOn _rom_call(BOOL,(void),5F7)
#define DateFormat_Get _rom_call(unsigned char,(void),5FC)
#define DateFormat_Set _rom_call(void,(unsigned char),5FD)
#define TimeFormat_Get _rom_call(unsigned char,(void),5FF)
#define TimeFormat_Set _rom_call(void,(unsigned char),600)

#define HS_MaxExpressions (*((unsigned short*)(_rom_call_addr_hack(5DE,\
       ((long)*((short *)_rom_call_addr(23C)+((AMS_1xx)?3:2))),204))))

// name of the backup file
#define BACKUP_FILE_NAME "bss_save"

typedef struct _backup
{
	unsigned char isclockon;
	unsigned char DateFormat;
	unsigned char TimeFormat;
	unsigned short MO_Options[20];
	unsigned short HSMaxExpressions;
	unsigned char contrast;
} BACKUP;



// ===============================================================
// prototyps
// ===============================================================
void RestoreSettings(void);
void BackupSettings(void);

unsigned char checksum(void* data, unsigned short bytes);



// main function
void _main(void)
{
	ESI         argptr;     // pointer to the arguments
	const char* arg;        // points to the first stinrg argument
	short       choice = 0; // choice of the user (popup)
	
	// init the argument pointer
	InitArgPtr(argptr);
	
	// we are just looking for string arguments
	if (GetArgType(argptr) == STR_TAG)
	{
		arg = GetStrnArg(argptr);
		
		if (strchr(arg, 'r'))
			RestoreSettings();
		else if (strchr(arg, 'b'))
			BackupSettings();
		
		if (strchr(arg, 'c'))
			DateAndTime_SetDlg();
		
		fatal(COMMENT_PROGRAM_NAME" "COMMENT_VERSION_STRING" by "COMMENT_AUTHORS);
	}
	
	// create a popup menu
	HANDLE h = PopupNew(COMMENT_PROGRAM_NAME" "COMMENT_VERSION_STRING, 0);
	
	// some error checking
	if (h == H_NULL) fatal("Out of mem");
	
	// add all texts
	PopupAddText(h, -1, "Backup settings", 0);
	PopupAddText(h, -1, "Restore settings", 0);
	PopupAddText(h, -1, "Exit", 0);
	
	// execute the popup
	choice = PopupDo(h, CENTER, CENTER, 0);
	
	// free <h>
	HeapFree(h);
	
	if (choice == 1) {
		BackupSettings();
		fatal("System settings saved!");
	}
	else if (choice == 2) {
		RestoreSettings();
		fatal("System settings restored!");
	}
	
	fatal(COMMENT_PROGRAM_NAME" "COMMENT_VERSION_STRING" by "COMMENT_AUTHORS);
}

void RestoreSettings(void)
{
	BACKUP backup;
	FILES file;
	EVENT ev;
	unsigned char chcksm;
	unsigned char backup_file_magic[strlen(BACKUP_FILE_MAGIC)+1];
	
	// unarchive the file
	EM_moveSymFromExtMem(SYMSTR(BACKUP_FILE_NAME), HS_NULL);
	
	// open the backup file
	if (FOpen(BACKUP_FILE_NAME, &file, FM_READ, "cfg") != FS_OK)
		fatal("Coudn't load the backup file!");
	
	// read the file magic
	backup_file_magic[strlen(BACKUP_FILE_MAGIC)] = 0;
	if (FRead((void*)backup_file_magic, strlen(BACKUP_FILE_MAGIC), &file) != FS_OK)
	{
		FClose(&file);
		fatal("Coudn't load the backup file!");
	}
	
	// check file version
	if (strcmp(BACKUP_FILE_MAGIC, backup_file_magic) != 0)
	{
		FClose(&file);
		fatal("Outdated file, create new backup!");
	}
	
	// read the file content
	if (FRead((void*)&backup, sizeof(BACKUP), &file) != FS_OK)
	{
		FClose(&file);
		fatal("Coudn't load the backup file!");
	}
	
	// read the checksum
	if (FRead((void*)&chcksm, 1, &file) != FS_OK)
	{
		FClose(&file);
		fatal("Coudn't load the backup file!");
	}
	
	// check checksum
	if (checksum((void*)&backup, sizeof(BACKUP)) != chcksm)
	{
		FClose(&file);
		fatal("Corrupted file, create new backup!");
	}
	
	// let the TSR's restore their settings
	ev.Type = CUSTOM_CM_RESTORE_SETTINGS_ALL_TSR;
	EV_sendEvent(TIOS_EV_getAppID("TIHOME"), &ev);
	
	// turn the clock on/off
	if (backup.isclockon)
		ClockOn();
	else
		ClockOff();
	
	HS_MaxExpressions = backup.HSMaxExpressions;
	
	// time and date format
	TRY
		DateFormat_Set(backup.DateFormat);
		TimeFormat_Set(backup.TimeFormat);
	ONERR
		FClose(&file);
		fatal("Error: Backup file may be corrupted!");
	ENDTRY
	
	// mode settings
	memcpy(ModeSettings, backup.MO_Options, 20*sizeof(short));
	MO_digestOptions(0);
	
	// contrast
	OSContrastValue = backup.contrast-1;
	OSContrastUp();
	
	// close the file
	FClose(&file);
	
	// archive the file
	EM_moveSymToExtMem(SYMSTR(BACKUP_FILE_NAME), HS_NULL);
}

void BackupSettings(void)
{
	BACKUP backup;
	FILES file;
	EVENT ev;
	
	// let the TSR's backup their settings
	ev.Type = CUSTOM_CM_BACKUP_SETTINGS_ALL_TSR;
	EV_sendEvent(TIOS_EV_getAppID("TIHOME"), &ev);
	
	// clock state
	if (IsClockOn()) backup.isclockon = 1;
	else             backup.isclockon = 0;
	
	backup.HSMaxExpressions = HS_MaxExpressions;
	
	// time and date format
	backup.DateFormat = DateFormat_Get();
	backup.TimeFormat = TimeFormat_Get();
	
	// mode settings
	MO_currentOptions();
	memcpy(backup.MO_Options, ModeSettings, 20*sizeof(short));
	
	// contrast
	backup.contrast = OSContrastValue;
	
	// unarchive the file
	EM_moveSymFromExtMem(SYMSTR(BACKUP_FILE_NAME), HS_NULL);
	
	// open the backup file
	if (FOpen(BACKUP_FILE_NAME, &file, FM_WRITE, "cfg") != FS_OK)
		fatal("Coudn't create backup file!");
	
	// write the file magic
	if (FWrite((void*)BACKUP_FILE_MAGIC, strlen(BACKUP_FILE_MAGIC), &file) != FS_OK)
	{
		FClose(&file);
		fatal("Coudn't write to backup file!");
	}
	
	// write backup
	if (FWrite((void*)&backup, sizeof(BACKUP), &file) != FS_OK)
	{
		FClose(&file);
		fatal("Coudn't write to backup file!");
	}
	
	// calculate checksum
	unsigned char chcksm = checksum((void*)&backup, sizeof(BACKUP));
	
	// write the checksum
	if (FWrite((void*)&chcksm, 1, &file) != FS_OK)
	{
		FClose(&file);
		fatal("Coudn't write to backup file!");
	}
	
	FClose(&file);
	
	// archive the file
	EM_moveSymToExtMem(SYMSTR(BACKUP_FILE_NAME), HS_NULL);
}

// calculates a simple 1-byte-checksum of the provided data
unsigned char checksum(void* data, unsigned short bytes)
{
	/*unsigned char res = 0;
	unsigned short i;
	
	for (i = 0; i < bytes; i++) res ^= ((unsigned char*)data)[i];
	
	return res;*/
	unsigned char res[16];
	MD5_CTX md5;
	
	// get MD5 hash of the data
	MD5Init(&md5);
	MD5Update(&md5, data, bytes);
	MD5Final(res, &md5);
	
	// we just use the first byte of the 16 byte long MD5 hash
	return res[0];
}

















